//  MIDImon Simple MIDI monitor program for PIC18F47K42 EasyPIC v8
//     - Put RX and TX UART swithes Tx on RC6, Rx on RC7
//     - Send MIDI data out RB5, receive MIDI data in on RB2
// use terminal program on PC set to 115200,8,n,1
// amateur code - use at your own risk - author accepts no responsibility for anything, anywhere, ever
// RAM usage is currently high - you may want to use PIC18F46K22 to expand this program - at least until text strings are moved to ROM
// connect RD6 Tx to MIDI out driver
// connect RD7 Rx to MIDI in receiver
// connect RC6 to MIDI driver circuitry for BassBoy MIDI connection - all BassBoy links fitted for chammel 1 synth
// MIDI details can be found on internet
// I use a "Sparkfun" MIDI shield as my MIDI in/out hardware interface - you will have to find you own way to convert 3.3v signals to/from MIDI

// status as of 30 Aug 2019....
// init of UART2 first else UART1 init kills UART2 output !!!!!
//
// commands b c d i in l and p serve no sensible purpose except maybe debug - strip out if you wish
// early version - will update as the mood takes me

// to do -
// fix backspace not working (and maybe others?) or just disable most control codes
// improve efficiency by re-using text and maybe store text as ROM
// read/write I2C EEPROM on-board to save MIDI settings and test files
// allow z command to call any test array, not just "midi_a"


// prototypes
void linein(char* pstring);
char getparams(char* tempstring);
void PrintHex(unsigned char i);
void PrintHexSp(unsigned char i);
void midi_off(void);

// function definitions
void cmdo(void);
void cmdp(void);
void cmdz(void);

// global variables
char rxbuff[20];                                                                // Buffer variable for storing data sent from master
char rxidx;                                                                     // Counter for data writen in buffer
unsigned char newcmd;
unsigned char cmd;
unsigned char u1_rx_data;
unsigned char u1_rx_flag = 0;
unsigned char u2_rx_data;
unsigned char u2_rx_flag = 0;
unsigned char linebuff[120];
unsigned char param[8][20];                                                     // up to 8 parameters
unsigned int upar[8];                                                           // parameters as word
unsigned char nparams;                                                          // number of parameters
unsigned char int_rx_buf[4];

// Bassboy test messages - see MIDI info online for MIDI format and notes before "cmdz" function - also BassBoy manual
//
unsigned char midi_a[] = {0xb0, 0x3a, 0x7f                                       // Control Change 58 accent 127
                            , 0xb0, 0x35, 0x7f , 0xb0, 0x36, 0x3f               // Filter pre gain, cut-off
                            , 0x90, 0x30, 0x7f , 0x90, 0x35, 0x7f               // note on 48, note on 53
                            , 0x90, 0x20, 0x7f , 0x90, 0x25, 0x7f               // note on 32, note on 35
                        };                                                      // end of note array

void interrupt() {                                                              // Interrupt routine
    if(U2RXIF_bit) {                                                            // Checks for Receive Interrupt Flag bit
        u2_rx_data = UART2_Read();                                              // Storing read data
        ByteToHex(u2_rx_data, int_rx_buf);
        u2_rx_flag = 1;
    }
}

void main() {
    rxidx  = 0;                                                                 // Initialisation of variable
    ANSELA = 0;                                                                 // Configure PORT pins as digital
    ANSELB = 0;                                                                 // Configure PORT pins as digital
    ANSELC = 0;                                                                 // Configure PORT pins as digital
    ANSELD = 0;                                                                 // Configure PORT pins as digital
    TRISA = 0;
    TRISB  = 0;                                                                 // Configure PORT as output
    TRISC  = 0;                                                                 // Configure PORT as output
    TRISD  = 0;                                                                 // Configure PORT as output
    LATA = 0;
    LATB = 0x00;                                                                // clear all to Low
    LATC = 0x00;                                                                // clear all to Low
    LATD = 0x00;                                                                // clear all to Low

    // initialise UART2 BEFORE UART1 - don't know why this is needed
    UART2_Init(31250);                                                          // Initialize UART 2 module for midi
    UART1_Init(115200);                                                         // Initialize UART 1 module at least 3x MIDI rate
    Delay_ms(100);                                                              // Wait for UART modules to stabilize
    newcmd = '?';                                                               // default to help
    cmd = newcmd;

    UART1_Write_Text("\r\n\nMIDI Monitor 30 Aug 2019\r\n");                     // introduction text
    UART1_Write_Text("MIDI Out RB5, MIDI In RB2 (via interface hardware)\r\n");                                         // introduction text
    UART1_Write_Text("? for help\r\n");                                         // introduction text

    U2RXIE_bit = 1;                                                             // turn ON interrupt on UART1 receive
    U2RXIF_bit = 0;                                                             // Clear interrupt flag
    //PEIE_bit  = 1;                                                              // Enable peripheral interrupts
    GIE_bit   = 1;                                                              // Enable GLOBAL interrupts

    while(1){
        linein(linebuff);                                                       // fetch a line of text
        newcmd = getparams(linebuff);                                           // read hex parameters
        switch(newcmd){                                                         // find command letter
          case 'O':
                cmdo();
                break;

            case 'P':
                cmdp();
                break;

            case 'Z':
                cmdz();
                break;

           case '?':
                UART1_Write_Text("\r\n?        List Commands");
                UART1_Write_Text("\r\no cc ..  Output hex to MIDI");
                UART1_Write_Text("\r\np        Parameter test");
                UART1_Write_Text("\r\nz        Bassboy send test\r\n");
                break;

            default:{
                UART1_Write_Text("\r\ncommand not recognised\r\n");
                break;
            }
        }
    }
}


// output bytes to MIDI
// needs parameter count <2 check
void cmdo(){
unsigned char pc;

    UART1_Write_Text("MIDI out ");
    for(pc = 1; pc < nparams; pc++){
        UART2_Write(upar[pc]);
        PrintHexSp(upar[pc]);
    }
    UART1_Write_Text("\r\n");
}

void cmdp(){
    unsigned char paramcount;
    PrintHex(nparams);
    UART1_Write_Text(" parameters\r\n");
    for( paramcount = 0 ; paramcount < 8 ; paramcount++ ){
        UART1_Write_Text("parameter ");
        PrintHexSp(paramcount);
        PrintHex((upar[paramcount] >> 8) & 0xff);
        PrintHex((upar[paramcount]) & 0xff);
        UART1_Write_Text("\r\n");
    }
}

// BassBoy MIDI send test
// insert all BassBoy links for channel 1 synth mode
// MIDI commnds include...
// 0x8x nn vv     Note off (velocity ignored)
// 0x9x nn vv     Note on
// 0xAx nn pp     Aftertouch (poly) pp=pressure
// 0xBx cc vv     Control Change vv=value
// 0xCx pp        Program Change pp=patch number
// 0xDx nn pp     Aftertouch (channel) pp=pressure
// 0xEx LL HH     Pitch Wheel llhh=pitch bend
// 0xFy ...       System Exclusive manufacturer defined
// x = MIDI channel 1 to 16
//
// 0xBx 07 vv     Volume (defsut 0x7f)
// 0xBx 35 vv     Filter pre-gain (default 0) 0=softest sound, 0x3f=hardest sound)
// 9xBx cc vv  see BassBoy manual for more details
//
// only notes 24 to 72 are recognised by BassBoy

void cmdz(){
unsigned char pc;

    UART1_Write_Text("\r\nBassBoy MIDI out on RB5\r\n");
    for(pc = 0; pc < sizeof(midi_a); pc++){
        UART2_Write(midi_a[pc]);
        if(midi_a[pc] > 127){                            // control byte
            Delay_ms(1000);
            UART1_Write_Text("\r\n");
        }
        PrintHexSp(midi_a[pc]);

    }
    midi_off();
    UART1_Write_Text("\r\n");
}

// turn off MIDI notes
void midi_off(){
    Delay_ms(2000);                                     // allow time to hear final note
    UART1_Write_Text("\r\nB0 7B 00 (all notes off)\r\n");
    UART2_Write(0xb0);
    UART2_Write(0x7b);
    UART2_Write(0x00);
    UART1_Write_Text("\r\ndone...\r\n");
}

// display byte in hex
void PrintHex(unsigned char i) {
unsigned char hi,lo;

    hi = i & 0xF0;                                                              // high nibble
    hi = hi >> 4;
    hi = hi + '0';
    if (hi > '9') hi = hi + 7;
    lo = (i & 0x0F) + '0';                                                      // low nibble
    if (lo>'9') lo=lo+7;

    UART1_Write(hi);
    UART1_Write(lo);
}

// display byte in hex with trailing space
void PrintHexSp(unsigned char i) {
unsigned char hi,lo;

    hi = i & 0xF0;                                                              // high nibble
    hi = hi >> 4;
    hi = hi + '0';
    if (hi > '9') hi = hi + 7;
    lo = (i & 0x0F) + '0';                                                      // low nibble
    if (lo>'9') lo=lo+7;

    UART1_Write(hi);
    UART1_Write(lo);
    UART1_Write(' ');
}

// get parameters from string
char getparams(char* tempstring){
char                 oldcmd;                                                    // remember old command letter
int                  n;                                                         // counter
char                 tempchar;                                                  // temp store
char                 tempchar2;                                                 // temp store
unsigned int         pointer1=0;                                                // pointer within string
unsigned int         pointer2=0;                                                // pointer within string

    oldcmd=cmd;                                                                 // remember old value
    nparams = 0;                                                                // initialise
    pointer1 = 0;                                                               // initialise

    for ( n = 0 ; n < 8 ; n++){                                                 // count 0 to 7
        // skip leading spaces
        tempchar=1;                                                             // init
        do {
            tempchar = tempstring[pointer1++];
        } while (tempchar == 0x20);
        pointer1--;                                                             // leave pointer after param

        // count parameters
        if ((tempchar != 0) && (nparams < 8)){
            nparams++;
        }

        if (n == 0){                                                            // only for param 0
            tempchar=tempstring[pointer1];                                      // get only first letter
            if ((tempchar > 0x60) && (tempchar < 0x7b)) {
                tempchar -= 0x20;                                               // to upper case
            }
            cmd=tempchar;
        }

        tempchar=1;                                                             // initialise
        if (nparams != 0){                                                      // only for non-blank line
            // get all characters
            pointer2=0;                                                         // initialise
            do {
                tempchar = tempstring[pointer1++];                              // get char
                if ((tempchar > 0x60) && (tempchar < 0x7b)) {
                    tempchar -= 0x20;                                           // to upper case
                }
                param[n][pointer2++] = tempchar;                                // and store it
            } while ((tempchar > 0x20) && (tempchar < 0x7f) && (pointer2 < 20)); // text
            param[n][pointer2-1]=0;                                             // zero terminate param string
            if((tempchar == 0)||(n == 9)){                                      // no more params if end of line
                break;
            }
        }
    }

    // convert parameter strings to hex longwords
    for (n = 0; n < nparams ; n++){                                             // parms are 0 to 7
        tempchar2=1;                                                            // initialise
        pointer1=0;                                                             // initialise
        upar[n]=0;                                                              // initialise
        do {
            tempchar = param[n][pointer1++];
            // read one character
            if ( tempchar != 0 ){
                upar[n] = upar[n] * 0x10;                                       // next most signicant hex digit
            }            tempchar2=tempchar;
                  // remember original character
            if ((tempchar < 0x30) || (tempchar > 0x46)) {
                tempchar = 0x00;                                                // non-hex to zero
            }
            if ((tempchar > 0x39) && (tempchar < 0x41)) {
                tempchar = 0x00;                                                // non-hex to zero
            }
            if ((tempchar > 0x40) && (tempchar < 0x47)) {
                tempchar -= 0x37;                                               // to bin
            }
            if ((tempchar > 0x2f) && (tempchar < 0x3a)) {
                tempchar -= 0x30;                                               // to bin
            }
            upar[n] += tempchar;
        } while (tempchar2 != 0);
    }

    if (nparams == 0) {
        cmd = oldcmd;
    }
    return cmd;
}


// get a string from UART1.
// Input: Buffer pointer.
// Output: updated buffer.
void linein(char* pstring){
char inChar = 0;

    while(inChar != '\r'){

        //UART1 read
        if (UART1_Data_Ready()){
            inChar = UART1_Read();
            UART1_Write(inChar);
            if(inChar == '\r'){
                UART1_Write('\n');
                *pstring ++= '\0';
            }
            else {
                if (inChar != 0x7f ){                                           // if not backspace
                    *pstring++ = inChar;
                }
                if (inChar == 0x7f ){                                           // handle backspace
                    *--pstring = 0x00;
                }
            }
        }
        if(u2_rx_flag){
            u2_rx_flag = 0;
            if(u2_rx_data > 127){
                UART1_Write_Text("\r\n");
                UART1_Write_Text(int_rx_buf);                                       //
            }
            else{
                UART1_Write_Text("-");
                UART1_Write_Text(int_rx_buf);                                       //
            }
        }
    }
}

// read a single charater from UART1
char chin(void){
char temp;

    while(!UART1_Data_Ready());
    temp = UART1_Read();
    return temp;
}

// write a single chc to UART1 and add lf to cr
void chout(char outchar){

    UART1_Write(outchar);
    if(outchar == 0x0d){
        UART1_Write(0x0a);
    }
}