//  PICmon8 Simple monitor program for PIC18F47K42 EasyPIC v8
//     - Put RX and TX UART swithes Tx on RC6, Rx on RC7
//     - Send MIDI data out RB5, receive MIDI data in on RB2
// use terminal program on PC set to 115200,8,n,1
// amateur code - use at your own risk - author accepts no responsibility for anything, anywhere, ever
// RAM usage is currently high - you may want to use PIC18F46K22 to expand this program - at least until text strings are moved to ROM

// status as of 30 Aug 2019....
// init of UART2 first else UART1 init kills UART2 output !!!!!
//
// commands b c d i in l and p serve no sensible purpose except maybe debug - strip out if you wish
// early version - will update as the mood takes me

// to do -
// fix backspace not working (and maybe others?) or just disable most control codes
// improve efficiency by re-using text and maybe store text as ROM

// mikroBUS 1 socket      // mikroBUS 2 socket      // mikroBUS 3 socket      // mikroBUS 4 socket      // mikroBUS 5 socket
// AN   RA2   RC0 PWM     // AN   RA3   RC1 PWM     // AN   RB3   RD3 PWM     // AN   RB3   RC2 PWM     // AN   RA2   RC0 PWM
// RST  RE1   RB0 INT     // RST  RA0   RB1 INT     // RST  RE2   RB2 INT     // RST  RA1   RB4 INT     // RST  RE1   RB0 INT
// CS   RE0   RC7 RX      // CS   RA5   RC7 RX      // CS   RD5   RD7 RX      // CS   RD2   RD7 RX      // CS   RA4   RC7 RX
// SCK  RC3   RC6 TX      // SCK  RC3   RC6 TX      // SCK  RD0   RD6 TX      // SCK  RD0   RD6 TX      // SCK  RC3   RC6 TX
// MISO RC4   RC3 SCL     // MISO RC4   RC3 SCL     // MISO RD1   RD0 SCL     // MISO RD1   RD0 SCL     // MISO RC4   RC3 SCL
// MOSI RC5   RC4 SDA     // MOSI RC5   RC4 SDA     // MOSI RD4   RD1 SDA     // MOSI RD4   RD1 SDA     // MOSI RC5   RC4 SDA
//      +3.3  +5          //      +3.3  +5          //      +3.3  +5          //      +3.3  +5          //      +3.3  +5
//      GND   GND         //      GND   GND         //      GND   GND         //      GND   GND         //      GND   GND

// EEPROM Click with  24C08-WP fitted
//     nc     WP       write protect when high - take low if write wanted - RC0 if mikroBUS1
//     nc     nc
//     nc     nc
//     nc     nc
//     nc     SCL      RC3 if mikroBUS1
//     nc     SDA      RC4 if mikroBUS1
//  +3.3V     +5V      (not used if 3.3V selected)
//    Gnd     Gnd

// prototypes
void linein(char* pstring);
char getparams(char* tempstring);
void PrintHex(unsigned char i);
void PrintHexSp(unsigned char i);
void midi_off(void);

// function definitions - for descriptions, see function code
void cmdf(void);
void cmdo(void);
void cmdp(void);
void cmdr(void);
void cmdw(void);

// global variables
char rxbuff[20];                                                                // Buffer variable for storing data sent from master
char rxidx;                                                                     // Counter for data writen in buffer
unsigned char newcmd;
unsigned char cmd;
unsigned char u1_rx_data;
unsigned char u1_rx_flag = 0;
unsigned char u2_rx_data;
unsigned char u2_rx_flag = 0;
unsigned char linebuff[120];
unsigned char param[8][20];                                                     // up to 8 parameters
unsigned int upar[8];                                                           // parameters as word
unsigned char nparams;                                                          // number of parameters
unsigned char int_rx_buf[4];
unsigned char EC_hdr;
unsigned char EC_tmp;
unsigned int EC_adr;
unsigned char EC_data;
unsigned char e_send[2];
unsigned int ee_addr;

void interrupt() {                                                              // Interrupt routine
    if(U2RXIF_bit) {                                                            // Checks for Receive Interrupt Flag bit
        u2_rx_data = UART2_Read();                                              // Storing read data
        ByteToHex(u2_rx_data, int_rx_buf);
        u2_rx_flag = 1;
    }
}

void main() {
    rxidx  = 0;                                                                 // Initialisation of variable
    ANSELA = 0;                                                                 // Configure PORT pins as digital
    ANSELB = 0;                                                                 // Configure PORT pins as digital
    ANSELC = 0;                                                                 // Configure PORT pins as digital
    ANSELD = 0;                                                                 // Configure PORT pins as digital
    TRISA = 0;
    TRISB  = 0;                                                                 // Configure PORT as output
    TRISC  = 0;                                                                 // Configure PORT as output
    TRISD  = 0;                                                                 // Configure PORT as output
    LATA = 0;
    LATB = 0x00;                                                                // clear all to Low
    LATC = 0x00;                                                                // clear all to Low
    LATD = 0x00;                                                                // clear all to Low

    // initialise UART2 BEFORE UART1 - don't know why this is needed
    UART2_Init(31250);                                                          // Initialize UART 2 module for midi
    UART1_Init(115200);                                                         // Initialize UART 1 module at least 3x MIDI rate
    Delay_ms(100);                                                              // Wait for UART modules to stabilize
    newcmd = '?';                                                               // default to help
    cmd = newcmd;

    UART1_Write_Text("\r\n\nPICmon8 Monitor 2 Sept 2019\r\n");                     // introduction text
    UART1_Write_Text("? for help\r\n");                                         // introduction text

    U2RXIE_bit = 1;                                                             // turn ON interrupt on UART1 receive
    U2RXIF_bit = 0;                                                             // Clear interrupt flag
    GIE_bit   = 1;                                                              // Enable GLOBAL interrupts

    while(1){
        linein(linebuff);                                                       // fetch a line of text
        newcmd = getparams(linebuff);                                           // read hex parameters
        switch(newcmd){                                                         // find command letter
            case 'F':
                cmdf();
                break;

            case 'O':
                cmdo();
                break;

            case 'P':
                cmdp();
                break;

            case 'R':
                cmdr();
                break;

            case 'W':
                cmdw();
                break;

           case '?':
                UART1_Write_Text("?        List Commands\r\n");
                UART1_Write_Text("f dd     Fill with dd EEPROM Click mikroBUS1\r\n");
                UART1_Write_Text("o cc ..  Output hex to MIDI\r\n");
                UART1_Write_Text("p        Parameter test\r\n");
                UART1_Write_Text("r        Read EEPROM Click mikroBUS1\r\n");
                UART1_Write_Text("w aaa dd Write EEPROM Click mikroBUS1\r\n");
                break;

            default:{
                UART1_Write_Text("\r\ncommand not recognised\r\n");
                break;
            }
        }
    }
}

// fill with dd EEPROM Click
// f dd      write data dd (hex) to address aaa (hex)
void cmdf(){

    e_send[1] = upar[1];

    LATC0_bit = 0;                                                              // WP write enable when low
    ANSELC0_bit = 0;                                                            // digital
    TRISC0_bit = 0;                                                             // output
    UART1_Write_Text("\r\nEEPROM Fill.");

    I2C1_Init();                                                                // initialize I2C communication

    Delay_ms(100);

    if(nparams == 2){
        for(ee_addr = 0; ee_addr < 1024; ee_addr++){
            EC_tmp = (ee_addr >> 8) & 0x03;                                     // get top 2 address bits
            EC_hdr = 0x50 + EC_tmp;                                             // bare EEPROM i2c address + hi 2 bits EEPROM aggress
            e_send[0]= ee_addr & 0x0ff;
            I2C1_Wr(EC_hdr, &e_send, 2, _I2C_END_MODE_STOP);                    // write byte
            Delay_ms(5);                                                        // do not try to write faster than EEPROM can hamdle
            if((ee_addr & 0x1f) == 0){
                UART1_Write_Text(".");
            }
        }
    }
    else{
        UART1_Write_Text("syntax error - need...\r\nf dd\r\n");
    }
    LATC0_bit = 1;                                                              // WP write protect when high
    UART1_Write_Text("\r\ndone\r\n");
}

// output bytes to MIDI
// needs parameter count <2 check
void cmdo(){
unsigned char pc;

    UART1_Write_Text("MIDI out ");
    for(pc = 1; pc < nparams; pc++){
        UART2_Write(upar[pc]);
        PrintHexSp(upar[pc]);
    }
    UART1_Write_Text("\r\n");
}

void cmdp(){
    unsigned char paramcount;
    PrintHex(nparams);
    UART1_Write_Text(" parameters\r\n");
    Delay_ms(100);                                                              // do not know why this is needed :(
    for( paramcount = 0 ; paramcount < 8 ; paramcount++ ){
        UART1_Write_Text("parameter ");
        PrintHexSp(paramcount);
        PrintHex((upar[paramcount] >> 8) & 0xff);
        PrintHex((upar[paramcount]) & 0xff);
        UART1_Write_Text("\r\n");
    }
}

// read EEPROM click in mikroBUS 1 socket
void cmdr(){

    LATC0_bit = 1;                                                              // WP write protect when high
    ANSELC0_bit = 0;                                                            // digital
    TRISC0_bit = 0;                                                             // output
    UART1_Write_Text("\r\nEEPROM Contents...");

    I2C1_Init();                                                                // initialize I2C communication
    Delay_ms(100);

    for(EC_adr = 0; EC_adr < 1024; EC_adr++){                                   // 1k bytes in EEPROM
        if(! (EC_adr & 0x0f)){                                                  // show address every 16th byte
            UART1_Write_Text("\r\n");
            PrintHex(EC_adr/256);
            PrintHex(EC_adr);
            UART1_Write_Text(" = ");
        }
        else{
            if(! (EC_adr & 0x03)){                                              // show address every 16th byte
                UART1_Write_Text("- ");
            }
        }
        EC_tmp = (EC_adr >> 8) & 0x03;                                          // get top 2 address bits
        EC_hdr = 0x50 + EC_tmp;                                                 // bare EEPROM i2c address + hi 2 bits EEPROM aggress
        I2C1_Wr(EC_hdr, &EC_adr, 1, _I2C_END_MODE_RESTART);                     // send byte (data address)
        I2C1_Rd(EC_hdr, &EC_data, 1, _I2C_END_MODE_STOP);                       // send byte (device address + R)
        PrintHexSp(EC_data);
        Delay_ms(1);                                                            // slow down the display rate
    }
    UART1_Write_Text("\r\n");
}

// write to EEPROM Click
// w aaa dd      write data dd (hex) to address aaa (hex)
void cmdw(){

    e_send[0] = upar[1];
    e_send[1] = upar[2];
    
    LATC0_bit = 0;                                                              // WP write enable when low
    ANSELC0_bit = 0;                                                            // digital
    TRISC0_bit = 0;                                                             // output
    UART1_Write_Text("\r\nEEPROM Write...");

    I2C1_Init();                                                                // initialize I2C communication

    Delay_ms(100);

    if(nparams == 3){
        EC_tmp = (upar[1] + 3) & 0x03;                                          // get top 2 address bits
        EC_hdr = 0x50 + EC_tmp;                                                 // bare EEPROM i2c address + hi 2 bits EEPROM aggress
        I2C1_Wr(EC_hdr, &e_send, 2, _I2C_END_MODE_STOP);                        // write byte

    }
    else{
        UART1_Write_Text("syntax error - need...\r\nw aaa dd\r\n");
    }
    LATC0_bit = 1;                                                              // WP write protect when high
    UART1_Write_Text("\r\n");
}

// display byte in hex
void PrintHex(unsigned char i) {
unsigned char hi,lo;

    hi = i & 0xF0;                                                              // high nibble
    hi = hi >> 4;
    hi = hi + '0';
    if (hi > '9') hi = hi + 7;
    lo = (i & 0x0F) + '0';                                                      // low nibble
    if (lo>'9') lo=lo+7;

    UART1_Write(hi);
    UART1_Write(lo);
}

// display byte in hex with trailing space
void PrintHexSp(unsigned char i) {
unsigned char hi,lo;

    hi = i & 0xF0;                                                              // high nibble
    hi = hi >> 4;
    hi = hi + '0';
    if (hi > '9') hi = hi + 7;
    lo = (i & 0x0F) + '0';                                                      // low nibble
    if (lo>'9') lo=lo+7;

    UART1_Write(hi);
    UART1_Write(lo);
    UART1_Write(' ');
}

// get parameters from string
char getparams(char* tempstring){
char                 oldcmd;                                                    // remember old command letter
int                  n;                                                         // counter
char                 tempchar;                                                  // temp store
char                 tempchar2;                                                 // temp store
unsigned int         pointer1=0;                                                // pointer within string
unsigned int         pointer2=0;                                                // pointer within string

    oldcmd=cmd;                                                                 // remember old value
    nparams = 0;                                                                // initialise
    pointer1 = 0;                                                               // initialise

    for ( n = 0 ; n < 8 ; n++){                                                 // count 0 to 7
        // skip leading spaces
        tempchar=1;                                                             // init
        do {
            tempchar = tempstring[pointer1++];
        } while (tempchar == 0x20);
        pointer1--;                                                             // leave pointer after param

        // count parameters
        if ((tempchar != 0) && (nparams < 8)){
            nparams++;
        }

        if (n == 0){                                                            // only for param 0
            tempchar=tempstring[pointer1];                                      // get only first letter
            if ((tempchar > 0x60) && (tempchar < 0x7b)) {
                tempchar -= 0x20;                                               // to upper case
            }
            cmd=tempchar;
        }

        tempchar=1;                                                             // initialise
        if (nparams != 0){                                                      // only for non-blank line
            // get all characters
            pointer2=0;                                                         // initialise
            do {
                tempchar = tempstring[pointer1++];                              // get char
                if ((tempchar > 0x60) && (tempchar < 0x7b)) {
                    tempchar -= 0x20;                                           // to upper case
                }
                param[n][pointer2++] = tempchar;                                // and store it
            } while ((tempchar > 0x20) && (tempchar < 0x7f) && (pointer2 < 20)); // text
            param[n][pointer2-1]=0;                                             // zero terminate param string
            if((tempchar == 0)||(n == 9)){                                      // no more params if end of line
                break;
            }
        }
    }

    // convert parameter strings to hex longwords
    for (n = 0; n < nparams ; n++){                                             // parms are 0 to 7
        tempchar2=1;                                                            // initialise
        pointer1=0;                                                             // initialise
        upar[n]=0;                                                              // initialise
        do {
            tempchar = param[n][pointer1++];
            // read one character
            if ( tempchar != 0 ){
                upar[n] = upar[n] * 0x10;                                       // next most signicant hex digit
            }            tempchar2=tempchar;
                  // remember original character
            if ((tempchar < 0x30) || (tempchar > 0x46)) {
                tempchar = 0x00;                                                // non-hex to zero
            }
            if ((tempchar > 0x39) && (tempchar < 0x41)) {
                tempchar = 0x00;                                                // non-hex to zero
            }
            if ((tempchar > 0x40) && (tempchar < 0x47)) {
                tempchar -= 0x37;                                               // to bin
            }
            if ((tempchar > 0x2f) && (tempchar < 0x3a)) {
                tempchar -= 0x30;                                               // to bin
            }
            upar[n] += tempchar;
        } while (tempchar2 != 0);
    }

    if (nparams == 0) {
        cmd = oldcmd;
    }
    return cmd;
}


// get a string from UART1.
// Input: Buffer pointer.
// Output: updated buffer.
void linein(char* pstring){
char inChar = 0;

    while(inChar != '\r'){

        //UART1 read
        if (UART1_Data_Ready()){
            inChar = UART1_Read();
            UART1_Write(inChar);
            if(inChar == '\r'){
                UART1_Write('\n');
                *pstring ++= '\0';
            }
            else {
                if (inChar != 0x7f ){                                           // if not backspace
                    *pstring++ = inChar;
                }
                if (inChar == 0x7f ){                                           // handle backspace
                    *--pstring = 0x00;
                }
            }
        }
        if(u2_rx_flag){
            u2_rx_flag = 0;
            if(u2_rx_data > 127){
                UART1_Write_Text("\r\n");
                UART1_Write_Text(int_rx_buf);                                   //
            }
            else{
                UART1_Write_Text("-");
                UART1_Write_Text(int_rx_buf);                                   //
            }
        }
    }
}

// read a single charater from UART1
char chin(void){
char temp;

    while(!UART1_Data_Ready());
    temp = UART1_Read();
    return temp;
}

// write a single chc to UART1 and add lf to cr
void chout(char outchar){

    UART1_Write(outchar);
    if(outchar == 0x0d){
        UART1_Write(0x0a);
    }
}