// EEPROM click simple test
// 24C08WP 8k 3.3v / 5V - address bits 0,0,0
// easyPIC 8 board mikroBUS1 socket
// PIC18F47K42 32MHz
//      - I2C address of EEPROM chip is 0xA2 (default).

// mikroBUS 1 socket
//       AN   RA2      RC0 PWM  WP 100k to gnd
//       RST  RE1      RB0 INT
//       CS   RE0      RC7 RX
//       SCK  RC3      RC6 TX
//       MISO RC4      RC3 SCL  SCL 4k7 to +3
//       MOSI RC5      RC4 SDA  SCL 4k7 to +3
//   vcc +3.3              +5
//       GND               GND

#define ARRAY_LENGTH 150

char someData[ARRAY_LENGTH];
char data_[10];
char nullData[ARRAY_LENGTH];
char rData[ARRAY_LENGTH];
char i ,tmpdata;


//--------------- Writes data to 24C02 EEPROM - signle location
void EEPROM_24C02_WrSingle(unsigned short wAddr, unsigned short wData) {
  data_[0] = wAddr;
  data_[1] = wData;
  I2C2_Remappable_Wr(0x50,data_,2,_I2C_END_MODE_STOP);
}//~

//--------------- Reads data from 24C02 EEPROM - single location (random)
unsigned short EEPROM_24C02_RdSingle(unsigned short rAddr) {
  data_[0] = rAddr;
  I2C2_Remappable_Wr(0x50,data_,1,_I2C_END_MODE_RESTART);
  I2C2_Remappable_Rd(0x50,data_,1,_I2C_END_MODE_STOP);

  return data_[0];
}

void main(){
  unsigned short i;

  ANSELA = 0;
  ANSELC = 0;
  ANSELD = 0;

  TRISC.B2 = 0;
  TRISA.B0 = 0;
  TRISA.B1 = 0;

  LATC.B2 = 0;

  PPS_Mapping(_RC3, _INPUT, _SCL2_in);
  PPS_Mapping(_RC4, _INPUT, _SDA2_in);

  PPS_Mapping(_RC4, _OUTPUT, _SCL2_out);
  PPS_Mapping(_RC4, _OUTPUT, _SDA2_out);

  ODCOND.B0 = 1;
  ODCOND.B1 = 1;

  I2C2_Remappable_Init();

  Delay_ms( 100 );
  memset(someData, 0x00, sizeof(someData));
  memset(rData, 0x01, sizeof(someData)); // something not zero

  LATA.B0 = 0;
  LATA.B1 = 0;

  for(i = 0; i < ARRAY_LENGTH; i++) {
    EEPROM_24C02_WrSingle(i,0x00);
    delay_ms(5);
  }

  for(i = 0x00; i < sizeof(someData); i++){
    rData[i] = EEPROM_24C02_RdSingle(i);
    delay_ms(5);
  }

  if (!memcmp(someData , rData, sizeof(someData))){
      LATA.B0 = 1;
  }

  for(i = 0; i < sizeof(someData); i++) {
    EEPROM_24C02_WrSingle(i,i);
    someData[i] = i;
    delay_ms(5);
  }

  for(i = 0x00; i < sizeof(someData); i++){
    rData[i] = EEPROM_24C02_RdSingle(i);
    delay_ms(5);
  }

  if (!memcmp(someData , rData, sizeof(someData))){
      LATA.B1 = 1;
  }
}