// GNSS 5 test program for EasyPIC8 and PIC47K42 at 32MHz
// see status and inprovement lists below
// may require that GNSS5 click be initialised using mikroE example code
//
// Connect PC (via hardware interface) to RC6 and RC7 with favourite terminal program 115200, 8, n, 1
// I use RS232 click board in mikroBUS 5 socket, but you can use on-board UART and RC6 Tx, RC7 Rx
// GNSS 5 Click board in mikroBUS 3 socket   (also works with older EasyGPS Click board, but GPS only hence, less satellites)

// Status 9 Sept 2019
// would like to fit LCD to easyPIC8, but any time I try to use LCD - something weird happens - beyond me, so I gave up on LCD
//
// improvements to do...
// show time on some kind of handy display device
// add GNSS 5 click setup code in case it has not been used before
// provide local time option - ideally adjusting summer/winter automatically using (20 year) date info

// indicator lights
// RA2 toggles when $GNGGA was last message to be received
// RA3 toggles when $GNGGA was last message to be received
// RA4 toggles when $GNGGA was last message to be received
// RA5 toggles when $GNGGA was last message to be received
// RB2 1 second pulse from GNSS5 click

// mikroBUS 1 socket      // mikroBUS 2 socket      // mikroBUS 3 socket      // mikroBUS 4 socket      // mikroBUS 5 socket
// AN   RA2   RC0 PWM     // AN   RA3   RC1 PWM     // AN   RB3   RD3 PWM     // AN   RB3   RC2 PWM     // AN   RA2   RC0 PWM
// RST  RE1   RB0 INT     // RST  RA0   RB1 INT     // RST  RE2   RB2 INT     // RST  RA1   RB4 INT     // RST  RE1   RB0 INT
// CS   RE0   RC7 RX      // CS   RA5   RC7 RX      // CS   RD5   RD7 RX      // CS   RD2   RD7 RX      // CS   RA4   RC7 RX
// SCK  RC3   RC6 TX      // SCK  RC3   RC6 TX      // SCK  RD0   RD6 TX      // SCK  RD0   RD6 TX      // SCK  RC3   RC6 TX
// MISO RC4   RC3 SCL     // MISO RC4   RC3 SCL     // MISO RD1   RD0 SCL     // MISO RD1   RD0 SCL     // MISO RC4   RC3 SCL
// MOSI RC5   RC4 SDA     // MOSI RC5   RC4 SDA     // MOSI RD4   RD1 SDA     // MOSI RD4   RD1 SDA     // MOSI RC5   RC4 SDA
//      +3.3  +5          //      +3.3  +5          //      +3.3  +5          //      +3.3  +5          //      +3.3  +5
//      GND   GND         //      GND   GND         //      GND   GND         //      GND   GND         //      GND   GND

// GNSS 5 Click    NEO-M8N
// mikroBUS socket 3 on EasyPIC 8
//        nc      RD3 i  EXTINT
//  RESET# i RE2  RB2 o  PULSE (1 second)
//        nc      RD7 0  TXD
//        nc      RD6 i  RXD
//        nc      RD0 i  SCL
//        nc      RD1 io SDA
//     +3.3V          +5V      (not used)
//       Gnd          Gnd

// prototypes
void linein(char* pstring);                                                     // read a line of text and process regular tasks
char getparams(char* tempstring);                                               // read and convert hexadecimal command parameters
void PrintHex(unsigned char i);                                                 // print a byte as hexadecimal
void PrintHexSp(unsigned char i);                                               // print hex followed by space
void get_field(unsigned char buffernum, unsigned char fieldnum);                // get a text field

// function definitions - for descriptions, see function code
void cmdm(void);                                                                // no raw printout
void cmdo(void);                                                                // output bytes to GNSS5
void cmdp(void);                                                                // debug only - parameter test
void cmdt(void);                                                                // toggle time display on/off
void cmdv(void);                                                                // turn on raw printout

// global variables
unsigned char GNGLL_ptr;                                                        // pointer to buffer containing GNGLL data
unsigned char GNGLL_flag;                                                       // GNGLL found
unsigned char GNRMC_ptr;                                                        // pointer to buffer containing GNRMC data
unsigned char GNRMC_flag;                                                       // GNRMC found
unsigned char GNVTG_ptr;                                                        // pointer to buffer containing GNVTG data
unsigned char GNVTG_flag;                                                       // GNVTG found
unsigned char GNGGA_ptr;                                                        // pointer to buffer containing GNGGA data
unsigned char GNGGA_flag;                                                       // GNGGA found
unsigned char buff_num;                                                         // buffer selection
unsigned char gnsspointer;                                                      // pointer for gnssbuff
unsigned char gnssbuff[16][256];                                                // buffer for characters received from UART2 GNSS5
unsigned char rxidx;                                                            // Counter for data writen in buffer
unsigned char newcmd;
unsigned char cmd;
unsigned char u1_rx_data;
unsigned char u1_rx_flag = 0;
unsigned char u2_rx_data;
unsigned char u2_rx_flag = 0;
unsigned char linebuff[120];
unsigned char param[8][20];                                                     // up to 8 parameters
unsigned int upar[8];                                                           // parameters as word
unsigned char nparams;                                                          // number of parameters
unsigned char int_rx_buf[4];
unsigned char EC_hdr;
unsigned char EC_tmp;
unsigned int EC_adr;
unsigned char EC_data;
unsigned char e_send[2];
unsigned int ee_addr;
unsigned char gprint_flag;
unsigned char t_flag;
unsigned char time_text[20];
unsigned char sat_text[2];                                                      // number of satellites
unsigned char field_text[40];                                                   // general purpose field text

void interrupt() {                                                              // Interrupt routine
    if(U2RXIF_bit) {                                                            // Checks for Receive Interrupt Flag bit
        u2_rx_data = UART2_Read();                                              // Storing read data
        int_rx_buf[0] = u2_rx_data;
        int_rx_buf[1] = 0;
        if(int_rx_buf[0] == '$'){
            if( gnssbuff[buff_num][1] == 'G'){
                if( gnssbuff[buff_num][5] == 'A'){
                    if(gnssbuff[buff_num][4] == 'G'){
                        GNGGA_ptr = buff_num;
                        LATA2_bit ^= 1;                                         // debug
                        GNGGA_flag = 1;
                    }
                }
                if( gnssbuff[buff_num][5] == 'C'){
                    GNRMC_ptr = buff_num;
                    LATA3_bit ^= 1;                                             // debug
                    GNRMC_flag = 1;
                }
                if( gnssbuff[buff_num][5] == 'G'){
                    GNVTG_ptr = buff_num;
                    LATA4_bit ^= 1;                                             // debug
                    GNVTG_flag = 1;
                }
                if( gnssbuff[buff_num][5] == 'L'){
                    GNGLL_ptr = buff_num;
                    LATA5_bit ^= 1;                                             // debug
                    GNGLL_flag = 1;
                }
            }
            gnsspointer = 0;
            buff_num++;
        }
        buff_num &= 0x0f;                                                       // stay within 16 buffer limit
        gnssbuff[buff_num][gnsspointer] = int_rx_buf[0];
        gnsspointer++;
        u2_rx_flag = 1;
    }
}

void main() {
    rxidx  = 0;                                                                 // Initialisation of variable
    ANSELA = 0;                                                                 // Configure PORT pins as digital
    ANSELB = 0;                                                                 // Configure PORT pins as digital
    ANSELC = 0;                                                                 // Configure PORT pins as digital
    ANSELD = 0;                                                                 // Configure PORT pins as digital
    ANSELE = 0;                                                                 // Configure PORT pins as digital
    TRISA = 0;
    TRISB  = 0x00;                                                              // Configure PORT direction
    TRISC  = 0x80;                                                              // RC7 input for UART, rest output
    TRISD  = 0x80;                                                              // RD7 input for UART, rest output
    TRISE  = 0;                                                                 // Configure PORT as output
    LATA = 0x02;                                                                // GNSS RESET# pin high
    LATB = 0x00;                                                                // clear all to Low
    LATC = 0x00;                                                                // clear all to Low
    LATD = 0x00;                                                                // clear all to Low
    LATE = 0x04;                                                                //
    
    // initialise UART2 BEFORE UART1 - don't know why this is needed
    UART2_Init(9600);                                                           // Initialize UART 2 module for GNSS
    UART1_Init(115200);                                                         // Initialize UART 1 module at least 3x MIDI rate
    Delay_ms(100);                                                              // Wait for UART modules to stabilize

    // it seems to be that UART init must be done before PPS mapping - don't know why

    Unlock_IOLOCK();
    PPS_Mapping_NoLock(0x1f, _INPUT, _RX2);                                     // map UART2 Rx to D7 (only ports B and C are valid for U2Rx)
    PPS_Mapping_NoLock(0x1e, _OUTPUT, _TX2PPS);                                 // map UART2 Tx to D6
    Delay_ms(200);                                                              // settle time

    gprint_flag = 0;
    t_flag = 1;
    newcmd = '?';                                                               // default to help
    cmd = newcmd;
    gnsspointer = 0;                                                            // initialise

    UART1_Write_Text("\r\n\nGNSS Monitor 9 Sept 2019\r\n");                     // introduction text
    UART1_Write_Text("? for help\r\n");                                         // introduction text

    U2RXIE_bit = 1;                                                             // turn ON interrupt on UART1 receive
    U2RXIF_bit = 0;                                                             // Clear interrupt flag
    GIE_bit   = 1;                                                              // Enable GLOBAL interrupts

    GNGLL_ptr = 0xff;                                                           // pointer to buffer containing GNGLL data
    GNRMC_ptr = 0xff;                                                           // pointer to buffer containing GNRMC data
    GNVTG_ptr = 0xff;                                                           // pointer to buffer containing GNVTG data
    buff_num = 0;
    
    while(1){
        linein(linebuff);                                                       // fetch a line of text
        newcmd = getparams(linebuff);                                           // read hex parameters
        switch(newcmd){                                                         // find command letter
            case 'M':
                cmdm();
                break;

            case 'O':
                cmdo();
                break;

            case 'P':
                cmdp();
                break;

            case 'T':
                cmdt();
                break;

            case 'V':
                cmdv();
                break;

           case '?':
                UART1_Write_Text("?        List Commands\r\n");
                UART1_Write_Text("m        message buffer print\r\n");
                UART1_Write_Text("o cc ..  Output hex to GNSS5\r\n");
                //UART1_Write_Text("p        Parameter test - debug only\r\n");
                UART1_Write_Text("t        toggle Time display on/off\r\n");
                UART1_Write_Text("v        toggle View raw GNSS text\r\n");
                break;

            default:{
                UART1_Write_Text("\r\ncommand not recognised\r\n");
                break;
            }
        }
    }
}

// message buffer printout
void cmdm(){
  unsigned char b_num;
  unsigned char b_ptr;
  unsigned char b_chc;
  unsigned char b_flag;
  unsigned char b_count;

    GNGLL_flag = 0;                                                             // clear old flag
    while(!GNGLL_flag);                                                         // wait for GNGLL message update to happen

    UART1_Write_Text("\r\nNewest buffer is 0x");
    PrintHexSp(buff_num);                                                       // print buffer number
    UART1_Write_Text("   (Oldest buffer shown first)");
    b_num = buff_num;
    
    for(b_count = 0; b_count < 16; b_count++){                                    // 16 buffers 0x00 to 0x0f
        b_num++;                                                                // print oldest buffer first
        if(b_num > 15) b_num = 0;                                               // wrap-around
        UART1_Write_Text("\r\n\nBuffer 0x");
        PrintHexSp(b_num);                                                      // print buffer number
        b_flag = 1;                                                             // enable printing
        for( b_ptr = 0; b_ptr < 255; b_ptr++){                                  // up to 255 characters
            if(b_flag){                                                         // check that print still wanted
                b_chc = GNSSbuff[b_num][b_ptr];
                UART1_Write(b_chc);
                if(b_chc == 0x0d){                                              // look for end of string
                    b_flag = 0;                                                 // and prevent rest of buffer from printing
                }
            }
        }
    }
    UART1_Write_Text("\r\n\n");
}

// output bytes to GNSS
// needs parameter count <2 check
void cmdo(){
unsigned char pc;

    UART1_Write_Text("GNSS out ");
    for(pc = 1; pc < nparams; pc++){                                            // count through parameters
        UART2_Write(upar[pc]);                                                  // send byte to UART2
        PrintHexSp(upar[pc]);                                                   // send hex info to consol
    }
    UART1_Write_Text("\r\n");
}

// no useful purpose - just a debug check
void cmdp(){
    unsigned char paramcount;
    PrintHex(nparams);
    UART1_Write_Text(" parameters\r\n");
    Delay_ms(100);                                                              // do not know why this is needed :(
    for( paramcount = 0 ; paramcount < 8 ; paramcount++ ){
        UART1_Write_Text("parameter ");
        PrintHexSp(paramcount);
        PrintHex((upar[paramcount] >> 8) & 0xff);
        PrintHex((upar[paramcount]) & 0xff);
        UART1_Write_Text("\r\n");
    }
}

// toggle time display on/off
void cmdt(){
    t_flag ^= 1;                                                                // toggle a flag - that's it
}

// enable raw GNSS print
void cmdv(){

    gprint_flag ^= 1;                                                           // toggle a flag - that is all
}

// display byte in hex
void PrintHex(unsigned char i) {
unsigned char hi,lo;

    hi = i & 0xF0;                                                              // high nibble
    hi = hi >> 4;
    hi = hi + '0';
    if (hi > '9') hi = hi + 7;
    lo = (i & 0x0F) + '0';                                                      // low nibble
    if (lo>'9') lo=lo+7;

    UART1_Write(hi);
    UART1_Write(lo);
}

// display byte in hex with trailing space
void PrintHexSp(unsigned char i) {
unsigned char hi,lo;

    hi = i & 0xF0;                                                              // high nibble
    hi = hi >> 4;
    hi = hi + '0';
    if (hi > '9') hi = hi + 7;
    lo = (i & 0x0F) + '0';                                                      // low nibble
    if (lo>'9') lo=lo+7;

    UART1_Write(hi);
    UART1_Write(lo);
    UART1_Write(' ');
}

// get parameters from string
char getparams(char* tempstring){
char                 oldcmd;                                                    // remember old command letter
int                  n;                                                         // counter
char                 tempchar;                                                  // temp store
char                 tempchar2;                                                 // temp store
unsigned int         pointer1=0;                                                // pointer within string
unsigned int         pointer2=0;                                                // pointer within string

    oldcmd=cmd;                                                                 // remember old value
    nparams = 0;                                                                // initialise
    pointer1 = 0;                                                               // initialise

    for ( n = 0 ; n < 8 ; n++){                                                 // count 0 to 7
        // skip leading spaces
        tempchar=1;                                                             // init
        do {
            tempchar = tempstring[pointer1++];
        } while (tempchar == 0x20);
        pointer1--;                                                             // leave pointer after param

        // count parameters
        if ((tempchar != 0) && (nparams < 8)){
            nparams++;
        }

        if (n == 0){                                                            // only for param 0
            tempchar=tempstring[pointer1];                                      // get only first letter
            if ((tempchar > 0x60) && (tempchar < 0x7b)) {
                tempchar -= 0x20;                                               // to upper case
            }
            cmd=tempchar;
        }

        tempchar=1;                                                             // initialise
        if (nparams != 0){                                                      // only for non-blank line
            // get all characters
            pointer2=0;                                                         // initialise
            do {
                tempchar = tempstring[pointer1++];                              // get char
                if ((tempchar > 0x60) && (tempchar < 0x7b)) {
                    tempchar -= 0x20;                                           // to upper case
                }
                param[n][pointer2++] = tempchar;                                // and store it
            } while ((tempchar > 0x20) && (tempchar < 0x7f) && (pointer2 < 20)); // text
            param[n][pointer2-1]=0;                                             // zero terminate param string
            if((tempchar == 0)||(n == 9)){                                      // no more params if end of line
                break;
            }
        }
    }

    // convert parameter strings to hex longwords
    for (n = 0; n < nparams ; n++){                                             // parms are 0 to 7
        tempchar2=1;                                                            // initialise
        pointer1=0;                                                             // initialise
        upar[n]=0;                                                              // initialise
        do {
            tempchar = param[n][pointer1++];
            // read one character
            if ( tempchar != 0 ){
                upar[n] = upar[n] * 0x10;                                       // next most signicant hex digit
            }
            tempchar2=tempchar;                                                 // remember original character
            if ((tempchar < 0x30) || (tempchar > 0x46)) {
                tempchar = 0x00;                                                // non-hex to zero
            }
            if ((tempchar > 0x39) && (tempchar < 0x41)) {
                tempchar = 0x00;                                                // non-hex to zero
            }
            if ((tempchar > 0x40) && (tempchar < 0x47)) {
                tempchar -= 0x37;                                               // to bin
            }
            if ((tempchar > 0x2f) && (tempchar < 0x3a)) {
                tempchar -= 0x30;                                               // to bin
            }
            upar[n] += tempchar;
        } while (tempchar2 != 0);
    }

    if (nparams == 0) {                                                         // if just empty line, repeat previous command
        cmd = oldcmd;
    }
    return cmd;
}

// get a string from UART1 and also handle regular tasks while waiting for input
// Input: Buffer pointer.
// Output: updated buffer.
void linein(char* pstring){
char inChar = 0;
unsigned char tpnt;
unsigned char b_num;

    while(inChar != '\r'){

        //UART1 read
        if (UART1_Data_Ready()){
            inChar = UART1_Read();
            UART1_Write(inChar);
            if(inChar == '\r'){
                UART1_Write('\n');
                *pstring ++= '\0';
            }
            else {
                if (inChar != 0x7f ){                                           // if not backspace
                    *pstring++ = inChar;
                }
                if (inChar == 0x7f ){                                           // handle backspace
                    *--pstring = 0x00;
                }
            }
        }
        if(u2_rx_flag){                                                         // GNSS data has been received
            u2_rx_flag = 0;
            if(gprint_flag){                                                    // flag decides whether to print raw GNSS
                UART1_Write_Text(int_rx_buf);                                   //
            }
        }
        if(GNGLL_flag){                                                         // time is contained within $GNGLL message
            if(t_flag){                                                         // only print if t flag set
                if(!gprint_flag){                                               // no time print if raw data print
                    UART1_Write_Text("\rUTC ");                                 // warn that this is UTC, not local time
                    b_num = GNGLL_ptr;
                    get_field(b_num, 6);                                        // $GNGLL field 6 is 9 bytes time
                    UART1_Write_Text(field_text);

                    UART1_Write_Text(" ");                                      //
                    b_num = GNGLL_ptr;
                    get_field(b_num, 7);                                        // $GNGLL field 7 is valid indicator A or invalid V
                    UART1_Write_Text(field_text);

                    UART1_Write_Text(" ");
                    b_num = GNRMC_ptr;
                    get_field(b_num, 10);                                       // $GNRMC field 10 is date
                    UART1_Write_Text(field_text);

                    UART1_Write_Text(" nSAT ");
                    b_num = GNGGA_ptr;
                    get_field(b_num, 8);                                        // $GNGGA field 8 is number of satellites
                    UART1_Write_Text(field_text);
                    UART1_Write_Text(" ");
                }
            }
            GNGLL_flag = 0;                                                     // prevent multiple prints
            UART2_Write(0x12);                                                  // cludge to clear GNSS frame errors
        }
    }
}

// get a text field from a buffer
void get_field(unsigned char buffernum, unsigned char fieldnum){
  unsigned char b_pnt;
  unsigned char b_chr;
  unsigned char f_start;
  unsigned char f_end;
  unsigned char f_count;
  unsigned char f_pnt;
  unsigned char fs_flag;

    f_count = 0;                                                                // start assuming field 1 ($Gxxxx)
    f_start = 0;
    f_end = 0;
    f_pnt = 0;
    b_pnt = 0;
    fs_flag = 1;

    while(fs_flag){                                                             // search whole buffer
        b_chr = gnssbuff[buffernum][b_pnt];
        field_text[f_pnt] = b_chr;
        if(b_chr == ','){
            f_end = b_pnt;
            field_text[f_pnt] = 0;
            f_count++;
            f_pnt = 0;
            if(f_count >= fieldnum) fs_flag = 0;
        }
        else{
            f_pnt++;
            if (f_pnt > 38){
                fs_flag = 0;                                                    // check for field too long
                field_text[0] = 0;                                              // invalidate return string
            }
        }
        b_pnt++;
        if (b_pnt == 255){
            fs_flag = 0;                                                        // check for end of buffer
            field_text[0] = 0;                                                  // invalidate return string
        }
    }
}