// PIC18F47K42 EasyPIC v8  HS-PLL, 8MHz Crystal, 32MHz MCU Clock
// Rotary click board click
// mikroC PRO for PIC 7.5.0
//     - Place Rotary click board in the mikroBUS socket 1.
//     - Pull down RB0 on the EasyPIC 8 board.

// take great care with configuration bits - only works with legacy interrupts

// mikroBUS 1 socket
//  ECB  AN   RA2      RC0 PWM  ECA
//  RST# RST  RE1      RB0 INT  SW
//  CS#  CS   RE0      RC7 RX
//  SCK  SCK  RC3      RC6 TX
//  SDO  MISO RC4      RC3 SCL
//  SDI  MOSI RC5      RC4 SDA
//       +3.3              +5
//       GND               GND

sbit HC595_cs           at LATE0_bit;
sbit ENCA               at RC0_bit;
sbit ENCB               at RA2_bit;
sbit ENC_SW             at RB0_bit;
sbit HC595_cs_direction at TRISE0_bit;
sbit ENCA_direction     at TRISC0_bit;
sbit ENCB_direction     at TRISA2_bit;
sbit ENC_SW_direction   at TRISB0_bit;

unsigned int Led_State = 0x0001;
char ENCA_bit;
char ENCB_bit;
char ENC_Button_State = 0;

// ISR
void Interrupt(){
    // check for interrupt
    if (INT0IF_bit){
        INT0IF_bit = 0;                                     // clear Interrupt flag
        ENC_Button_State++;                                   // set new ENC button state
        if (ENC_Button_State == 8){
            ENC_Button_State = 0;
        }
        // check ENC button state
        switch (ENC_Button_State){
          case 0 : Led_State = 0x0001;                      // One LED are ON
                   break;
          case 1 : Led_State = 0x0101;                      // Two LEDs are ON
                   break;
          case 2 : Led_State = 0x1111;                      // Four LEDs are ON
                   break;
          case 3 : Led_State = 0x5555;                      // Eight LEDs are ON
                   break;
          case 4 : Led_State = 0x0003;                      // 2 LED are ON
                   break;
          case 5 : Led_State = 0x0007;                      // Three LEDs are ON
                   break;
          case 6 : Led_State = 0x1fff;                      // 15 LEDs are ON
                   break;
          case 7 : Led_State = 0x00ff;                      // Eight LEDs are ON
                   break;
           default :
                   break;
        }
    }
}

void HC595_Write(int led_value){
char first_byte;
char second_byte;
  
    second_byte = led_value;
    //value = value >> 8;
    first_byte = led_value >> 8;

    SPI1_write(first_byte);
    SPI1_write(second_byte);

    HC595_cs = 0;                                         // latch serial data to outputs
    asm{nop};
    asm{nop};
    asm{nop};
    HC595_cs = 1;
}

void main() {
    ANSELA = 0 ;                                          // Configure PORTA pins as digital
    ANSELB = 0 ;                                          // Configure PORTB pins as digital
    ANSELC = 0 ;                                          // Configure PORTC pins as digital
    ANSELD = 0 ;                                          // Configure PORTC pins as digital
    ANSELE = 0 ;                                          // Configure PORTC pins as digital
    TRISA = 0;
    TRISB = 0;
    TRISC = 0;
    TRISD = 0;
    TRISE = 0;                                            // all output
    LATA = 0;
    LATB = 0;
    LATC = 0;
    LATD = 0;
    LATE = 3;
    
    ENCA_direction     = 1;                               // Configure RC0 as input
    ENCB_direction     = 1;                               // Configure RA2 as input
    ENC_SW_direction   = 1;                               // Configure RB0 as input
    HC595_cs_direction = 0;                               // Configure RE0 as output

    INT0IE_bit = 1 ;                                      // Enable INTE interrupt on RB0
    GIE_bit = 1 ;                                         // Enable Global Interrupt

    SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);     // slow SPI clk essential
    
    while(1) {
        if((ENCA_bit != ENCA) || (ENCB_bit != ENCB)){   // If logic state on pin A or pin B change
                                                            // Compare last logic state of pin B and new logick state on pin A
            if(ENCB_bit | ENCA == 1){                          // If ENCB_old OR ENCA_new is 1
                Led_State = (Led_State << 1) | (Led_State >> 15);
            }

            if(ENCA_bit | ENCB == 1){                          // If ENCA_old OR ENCB_new is 1
                Led_State = (Led_State >> 1) | (Led_State <<15);
            }

            ENCA_bit = ENCA ;                              // Set new value of ENCA_bit
            ENCB_bit = ENCB ;                              // Set new value of ENCB_bit
        }

    HC595_Write(Led_State);
    }
}