// ******* Use of this code is entirely at user's risk *******
// This frequency counter is missing the ground/input isolation circuitry to protect against external high/low voltages
// External circuits often have voltages present that the user may not be aware of
// EasyPIC7 itself may have significant voltages present, such that a short to ground might cause damage
//
// simple frequency counter for EasyPIC7 with LCD fitted
// amateur code for fun only - no responsibility accepted for anything
// early verson - ultra simple
// do not put any voltage lower than 0 Volts or higher than Vcc into the input, or you will destroy your EasyPIC7
//
// use 8MHz crystal, HS oscillator, PLL (32 MHz processor clock), mikroC PRO for PIC v.5.2.0
// for PIC18F45K22 - turn on LEDs for PORTA to see 1 second toggle
// install a spare link from RC2 pin (PWM output) to RC0 pin (counter input) to provide a test frequency
// tested for frequencies up to 8MHz - may work up to 50MHz (datasheet is unclear, plus board design may have an effect)
// needs libraries selected - Conversions, C_String, Lcd, Lcd_Constants, PWM12
//
// feed your own square wave into RC0 and be sure to connect input ground to EasyPIC7 ground
// provide pull-up on PORT RE_0 for frequency display, put pull down for counter - reset to implement any change
// *** WARNING *** take care not to destroy your EasyPIC7 by applying any voltage less then 0V or more than Vcc
// maybe fit 1k resistor in series with RC0 input for extra safety
// *** disclaimer *** If you destroy your PIC or your board, I will not be resposible - connect external circuits at your own risk

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

// Pin direction
sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D7_Direction at TRISB3_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D4_Direction at TRISB0_bit;


void main() {
unsigned long frequency;               // complete frequency count
unsigned long disp_frequency;          // frequency count for display
unsigned long freq_low;                // low word of frequency count
char freq_txt[11];                     // buffer for ASCII conversion
char fc_flag;                          // non-zero for frequency counter, zero for counter

     ANSELA = 0;                       // Configure PORTA pins as digital
     ANSELB = 0;                       // Configure PORTB pins as digital
     ANSELC = 0;                       // Configure PORTC pins as digital
     ANSELD = 0;                       // Configure PORTD pins as digital
     ANSELE = 0;                       // Configure PORTE pins as digital
     ADCON0 = 0;                       // A/D off
     ADCON1 = 0x0f;                    // all digital
     CM1CON0 = 0;                      // disable 18F45K22 comparator
     CM2CON0 = 0;                      // disable 18F45K22 comparator

     LATA = 0;                         // all off
     LATB = 0xff;                      // disable GLCD if fitted
     LATC = 0;                         // all off
     TRISA = 0xC0;                     // all output except OSC pins
     TRISB = 0x00;                     // all output
     TRISC = 0x01;                     // all output except RC0
     TRISE = 0xff;                     // all input

     Delay_ms(100);                    // Wait for LCD to stabilise
     Lcd_Init();                          // Initialize Lcd
     Lcd_Cmd(_LCD_CLEAR);                 // Clear display
     Lcd_Cmd(_LCD_CURSOR_OFF);            // Turn cursor off
     if(RE0_bit){
         //Lcd_Out(1,1, "Frequency on RC0");    // Print text to Lcd, 1st row, 1st column
         Lcd_Out(2,1, "0000000000 Hz   ");    // Print text to Lcd, 2nd row, 1st column
         fc_flag = 1;                         // show that frequency counter selected
     }
     else{
         //Lcd_Out(1,1, "Count from RC0  ");    // Print text to Lcd, 1st row, 1st column
         Lcd_Out(2,1, "0000000000      ");    // Print text to Lcd, 2nd row, 1st column
         fc_flag = 0;                         // show that counter selected
     }
     
     PWM1_Init(2000);                  // Test frequency PWM1 module
     PWM1_Start();                     // start PWM1
     PWM1_Set_Duty(127 );              // Set current duty for PWM1 50%
     T0CON = 0x87;                     // T0 on, 16 bit, prescaler 256
     T1CON = 0x85;                     // T1 on, ext input, no prescale
     T1GCON = 0x00;                    // no gating
     
     frequency = 0;                // initialise count
     TMR1L = 0;                    // reset to 0
     TMR1H = 0;                    // reset to 0

    // this is the main loop, where everything happens
     while(1){
         TMR0H = 0x85;                 // set up timer 0 for 1 second time out
         TMR0L = 0xee;                 // writing TMR0L also writes TMR0H
         INTCON.TMR0IF = 0;            // reset time-out indicator
         if(fc_flag){                  // have we selected frequency rather than counter?
             frequency = 0;                // initialise count - comment out
             PIR1.TMR1IF = 0;              // initialise
             TMR1L = 0;                    // reset to 0
             TMR1H = 0;                    // reset to 0
         }

         while(!INTCON.TMR0IF){        // loop while waiting for 1 second time-out
             if(PIR1.TMR1IF){          // monitor frequency count overflow
                 PIR1.TMR1IF=0;        // reset overflow
                 frequency += 65536;   // increment count high bytes
             }
         }
         
         freq_low = (TMR1H << 8) + TMR1L;  // collect frequency bytes
         disp_frequency = frequency + freq_low;            // collect frequency bytes
         LongWordToStrWithZeros(disp_frequency, freq_txt);          // covert result to ASCII
         Lcd_Out(2,1, freq_txt);       // Print text to Lcd, 2nd row, 1st column
         LATA=LATA^1;                  // show some life by toggling LED
     }
}